import glob
import logging
import os.path as osp
import shutil


def copy_debug_authoring_plugin_to_release_dir(build_config, plugin_name):
    if build_config == 'Release':
        return

    wwise_root = osp.expandvars('%wwiseroot%')
    build_output_dir = osp.join(wwise_root, f'Authoring/x64/{build_config}/bin/Plugins')
    src_files = glob.glob(osp.join(build_output_dir, f'{plugin_name}.*'))
    dst_dir = osp.join(wwise_root, f'Authoring/x64/Release/bin/Plugins')
    logging.info(f'Copy authoring plugin "{src_files}", from "{build_output_dir}" to "{dst_dir}"')
    for src in src_files:
        dst = osp.join(dst_dir, osp.basename(src))
        shutil.copy(src, dst)


def main(**kwargs):
    """
    Will be called after build.
    kwargs:
        proj_root: project root path
        build_config: build configuration (Debug, Profile, Release)
        plugin_name: plugin name, which is used to glob plugin files
    """
    copy_debug_authoring_plugin_to_release_dir(kwargs['build_config'], kwargs['plugin_name'])


if __name__ == '__main__':
    _proj_root = osp.abspath(f'{__file__}/../../..')
    _plugin_name = osp.basename(_proj_root)
    # Just for manually run
    main(proj_root=_proj_root, build_config='Debug', plugin_name=_plugin_name)



import logging
import os.path as osp

import kkpyutil as util

from wpe.pathman import PathMan


class HookProcessor:
    def __init__(self, path_man: PathMan, build_config: str, target_hooks: list[str]):
        self.pathMan = path_man
        self.buildConfig = build_config
        self.targetHooks = target_hooks

    def process_pre_hook(self, command):
        self._process_hook('pre', command)

    def process_post_hook(self, command):
        self._process_hook('post', command)

    def _process_hook(self, phase, command):
        def should_execute(_hook_name):
            if not self.targetHooks or _hook_name in self.targetHooks:
                return osp.isfile(osp.join(self.pathMan.hooksDir, f'{hook_name}.py'))

        hook_name = f'{phase}_{command}'
        if not should_execute(hook_name):
            return

        hook_module = util.safe_import_module(hook_name, self.pathMan.hooksDir)
        logging.info(f'Running hook: {hook_name}')
        hook_module.main(proj_root=self.pathMan.root,
                         build_config=self.buildConfig,
                         plugin_name=self.pathMan.pluginName,
                         )



import os
import os.path as osp
import re

import kkpyutil as util


class PathMan:
    def __init__(self, cwd=None):
        self.templatesDir = osp.join(osp.dirname(__file__), 'templates')
        self.premakePluginLua = self.find_premake_plugin_lua_in_ancestor_and_update_root(cwd or os.getcwd())
        self.root = osp.dirname(self.premakePluginLua)
        os.chdir(self.root)
        self.pluginName = self.parse_plugin_name()
        self.pluginConfigHeader = osp.join(self.root, f'{self.pluginName}Config.h')
        self.pluginId = self.parse_plugin_id()
        self.configDir = osp.join(self.root, '.wpe')
        self.parameterConfig = osp.join(self.configDir, 'wpe_parameters.toml')
        self.docsDir = osp.join(self.root, 'WwisePlugin/res/Md')
        self.distDir = osp.join(self.root, 'dist')
        self.hooksDir = osp.join(self.configDir, 'hooks')
        self.buildNumberFile = osp.join(self.configDir, 'build_number')

    @staticmethod
    def find_premake_plugin_lua_in_ancestor_and_update_root(cwd):
        premake_script_filename = 'PremakePlugin.lua'
        while cwd:
            lua = osp.join(cwd, premake_script_filename)
            if osp.isfile(lua):
                return lua
            parent = osp.dirname(cwd)
            if parent == cwd:
                break
            cwd = parent
        raise FileNotFoundError(f'Can not find PremakePlugin.lua in ancestor directories. '
                                f'This command must be executed under a plugin directory. cwd: {os.getcwd()}')

    def parse_plugin_name(self):
        lines = util.load_lines(self.premakePluginLua, rmlineend=True)
        name_define_pattern = r'Plugin.name = ".*"'
        for line in lines:
            if matched := re.match(name_define_pattern, line):
                return matched.group().split('"')[1]

    def parse_plugin_id(self):
        lines = util.load_lines(self.pluginConfigHeader, rmlineend=True)
        prefix = '    static const unsigned short PluginID = '
        name_define_pattern = rf'{prefix}\d+'
        for line in lines:
            if matched := re.match(name_define_pattern, line):
                return int(matched.group().lstrip(prefix))



import copy
import logging
import os.path as osp
from typing import Any, Optional
from dataclasses import dataclass, field
import xml.etree.ElementTree as ET

import kkpyutil as util

# project
from wpe.util import *

_type_prefix_map = {
    'float': 'f',
    'int': 'i',
    'bool': 'b',
}

_wwise_type_name_map = {
    'float': 'AkReal32',
    'int': 'AkInt32',
    'bool': 'bool',
}

_xml_type_name_map = {
    'float': 'Real32',
    'int': 'int32',
    'bool': 'bool',
}


def auto_add_line_end(lines: list[str]):
    for i, line in enumerate(lines):
        if not line.endswith('\n'):
            lines[i] += '\n'
    return lines


@dataclass
class Parameter:
    name: str
    type_: str
    rtpc: bool
    defaultValue: Any
    minValue: Any
    maxValue: Any
    description: list[dict] = field(default_factory=list)
    dependencies: list[dict] = field(default_factory=list)
    displayName: str = ''
    enumeration: list[dict] = field(default_factory=list)
    userInterface: dict = field(default_factory=dict)
    id: int = 0

    def __post_init__(self):
        if self.type_ == 'bool':
            self.defaultValue = str(self.defaultValue).lower()
            self.minValue = 'false'
            self.maxValue = 'true'
        self.propertyName = util.convert_compound_cases(self.name)
        self.cppVariableName = _type_prefix_map[self.type_] + self.propertyName
        self.paramIDName = f'PARAM_{self.name.upper()}_ID'
        self.typeName = _wwise_type_name_map[self.type_]
        self.xmlTypeName = _xml_type_name_map[self.type_]
        self.struct = 'RTPC' if self.rtpc else 'NonRTPC'
        self.displayName = self.displayName or util.convert_compound_cases(self.name, style='title')

    def assign_id(self, _id: int):
        self.id = _id

    @staticmethod
    def create(name, dict_define: dict[str, Any]):
        return Parameter(
            name=name,
            type_=dict_define['type'],
            rtpc=dict_define['rtpc'],
            defaultValue=dict_define['default_value'],
            minValue=dict_define.get('min_value', None),
            maxValue=dict_define.get('max_value', None),
            description=dict_define.get('description', []),
            dependencies=dict_define.get('dependencies', []),
            displayName=dict_define.get('display_name', ''),
            enumeration=dict_define.get('enumeration', []),
            userInterface=dict_define.get('user_interface', '')
        )

    def generate_param_id(self) -> str:
        return f'static constexpr AkPluginParamID {self.paramIDName} = {self.id};'

    def generate_declaration(self) -> str:
        return f'{self.typeName} {self.cppVariableName};'

    def generate_init(self) -> str:
        return f'{self.struct}.{self.cppVariableName} = {self.defaultValue};'

    def generate_read_bank_data(self) -> str:
        return f'{self.struct}.{self.cppVariableName} = READBANKDATA({self.typeName}, pParamsBlock, in_ulBlockSize);'

    def generate_set_parameter(self) -> str:
        need_reinterpret = self.typeName != 'AkReal32' and self.rtpc
        interpret_pointer = f'static_cast<{self.typeName}>(*(AkReal32*)in_pValue)' if need_reinterpret else f'*(({self.typeName}*)in_pValue)'
        return f'''    case {self.paramIDName}:
        {self.struct}.{self.cppVariableName} = {interpret_pointer};
        m_paramChangeHandler.SetParamChange({self.paramIDName});
        break;'''

    def generate_property_name_declaration(self) -> str:
        return f'extern const char* const sz{self.propertyName};'

    def generate_property_name_definition(self) -> str:
        return f'const char* const sz{self.propertyName} = "{self.propertyName}";'

    def generate_write_bank_data(self) -> str:
        writer = 'Write' + util.convert_compound_cases(self.typeName.lstrip('Ak'))
        getter = 'Get' + util.convert_compound_cases(self.typeName.lstrip('Ak'))
        return f'in_dataWriter.{writer}(m_propertySet.{getter}(in_guidPlatform, sz{self.propertyName}));'

    def generate_parameter_gui(self) -> list[str]:
        def _generate_dependencies():
            if not self.dependencies:
                return ''
            dependencies_element = ET.Element('Dependencies')
            for dep in self.dependencies:
                property_dep = ET.SubElement(dependencies_element, 'PropertyDependency')
                property_dep.attrib['Name'] = dep['obj'].propertyName
                property_dep.attrib['Action'] = 'Enable'
                condition = ET.SubElement(property_dep, 'Condition')
                if (condition_value := dep['condition']) == 'Enumeration':
                    enumeration = ET.SubElement(condition, condition_value)
                    enumeration.attrib['Type'] = dep['obj'].xmlTypeName
                    for value in dep['values']:
                        value_tag = ET.SubElement(enumeration, 'Value')
                        value_tag.text = str(value)
                if (condition_value := dep['condition']) == 'Range':
                    range_ = ET.SubElement(condition, condition_value)
                    range_.attrib['Type'] = dep['obj'].xmlTypeName
                    min_value = ET.SubElement(range_, 'Min')
                    min_value.text = str(dep['min'])
                    max_value = ET.SubElement(range_, 'Max')
                    max_value.text = str(dep['max'])
            ET.indent(dependencies_element)
            return '\n' + ET.tostring(dependencies_element).decode(util.TXT_CODEC)
        support_rtpc_type = 'SupportRTPCType="Exclusive"' if self.rtpc else ''

        def _generate_bool_gui_lines():
            return f'''<Property Name="{self.propertyName}" Type="{self.xmlTypeName}" {support_rtpc_type} DisplayName="{self.displayName}">
  <DefaultValue>{self.defaultValue}</DefaultValue>
  <AudioEnginePropertyID>{self.id}</AudioEnginePropertyID>{_generate_dependencies()}
</Property>'''.splitlines()

        def _generate_int_gui_lines():
            user_interface = self.userInterface
            if not self.enumeration:
                raise ValueError(f'Parameter "{self.name}" is int type but no enumeration provided. Please provide enumeration field with string list.')
            options = '\n        '.join(['<Value DisplayName="{}">{}</Value>'.format(opt['displayName'], opt['value']) for opt in self.enumeration])
            return f'''<Property Name="{self.propertyName}" Type="{self.xmlTypeName}" {support_rtpc_type} DisplayName="{self.displayName}">
  <UserInterface {user_interface} />
  <DefaultValue>{self.defaultValue}</DefaultValue>
  <AudioEnginePropertyID>{self.id}</AudioEnginePropertyID>
  <Restrictions>
    <ValueRestriction>
      <Enumeration Type="{self.xmlTypeName}"> 
        {options}
      </Enumeration>
    </ValueRestriction>
  </Restrictions>{_generate_dependencies()}
</Property>'''.splitlines()

        def _generate_float_gui_lines():
            user_interface = self.userInterface or f'Step="0.1" Fine="0.001" Decimals="3" UIMax="{self.maxValue}"'
            return f'''<Property Name="{self.propertyName}" Type="{self.xmlTypeName}" {support_rtpc_type} DisplayName="{self.displayName}">
  <UserInterface {user_interface} />
  <DefaultValue>{self.defaultValue}</DefaultValue>
  <AudioEnginePropertyID>{self.id}</AudioEnginePropertyID>
  <Restrictions>
    <ValueRestriction>
      <Range Type="{self.xmlTypeName}">
        <Min>{self.minValue}</Min>
        <Max>{self.maxValue}</Max>
      </Range>
    </ValueRestriction>
  </Restrictions>{_generate_dependencies()}
</Property>'''.splitlines()

        if self.type_ == 'bool':
            return _generate_bool_gui_lines()

        if self.type_ == 'int':
            return _generate_int_gui_lines()

        if self.type_ == 'float':
            return _generate_float_gui_lines()

        raise NotImplementedError(f'Parameter type "{self.type_}" not supported.')

    def dump_parameter_doc(self, docs_dir: str):
        for lang in self.description:
            output_path = osp.join(docs_dir, f'{lang["language"]}', f'{self.propertyName}.md')
            doc_str = f'''##{self.displayName}

{lang['text']}

Range: {self.minValue} - {self.maxValue} <br/>'''
            util.save_text(output_path, doc_str)


class PluginInfo:
    def __init__(self, info_dict: dict):
        self.infoDict = info_dict
        self.defaultPlatformSupport = '''<Platform Name="Any">
  <CanBeInsertOnBusses>true</CanBeInsertOnBusses>
  <CanBeInsertOnAudioObjects>true</CanBeInsertOnAudioObjects>
  <CanBeRendered>true</CanBeRendered>
</Platform>'''

    def generate_platform_support(self) -> list[str]:
        if not self.infoDict['platform_support']:
            return self.defaultPlatformSupport.splitlines()
        lines = []
        for platform, config in self.infoDict['platform_support'].items():
            lines.append(f'<Platform Name="{platform}">')
            for key, value in config.items():
                lines.append(f'  <{key}>{str(value).lower()}</{key}>')
            lines.append('</Platform>')
        return lines


class ParameterGenerator:
    def __init__(self, path_man, is_forced=False):
        self.pathMan = path_man
        self.isForced = is_forced
        self.parameters: dict[str, Parameter] = {}
        self.pluginInfo: Optional[PluginInfo] = None

    def main(self):
        self._load_parameter_config()
        self._generate()

    def _load_parameter_config(self):
        if not osp.isfile(self.pathMan.parameterConfig):
            raise FileNotFoundError(f'Parameter config not found: {self.pathMan.parameterConfig}')
        content = load_toml(self.pathMan.parameterConfig)

        # load plugin info
        self.pluginInfo = PluginInfo(content['plugin_info'])

        # load parameters
        if 'parameters' not in content:
            return
        for name, define in content['parameters'].get('defines', {}).items():
            self.parameters[name] = Parameter.create(name, define)
        for instance in content['parameters'].get('from_templates', []):
            template = copy.deepcopy(content['templates'][instance['template']])
            for key, value in instance.get('override', {}).items():
                template[key] = value
            if 'dependencies' in template:
                for dep in template['dependencies']:
                    dep['name'] = dep['name'] % {'suffix': instance['suffix']}

            name = f"{instance['template']}_{instance['suffix']}"
            self.parameters[name] = Parameter.create(name, template)

        for i, param in enumerate(self.parameters.values()):
            param.assign_id(i)
            for dep in param.dependencies:
                dep['obj'] = self.parameters[dep['name']]

    def _generate(self):
        def _copy_template(relative):
            def _need_overwrite(_dst):
                if self.isForced:
                    return True
                if osp.isfile(_dst):
                    content = util.load_text(_dst)
                    return '[wp-enhanced template]' not in content
                return True

            src = osp.join(self.pathMan.templatesDir, relative)
            dst = replace_in_basename(osp.join(self.pathMan.root, relative), 'ProjectName', self.pathMan.pluginName)
            if not _need_overwrite(dst):
                logging.info(f'Skip copying template "{osp.basename(src)}". Use -f to force overwrite.')
                return dst
            if osp.isfile(src):
                overwrite_copy(src, dst)
                util.substitute_keywords_in_file(dst,
                                                 {
                                                     'name': self.pathMan.pluginName,
                                                     'display_name': self.pathMan.pluginName,
                                                     'plugin_id': self.pathMan.pluginId,
                                                 })
                return dst
            else:
                raise FileNotFoundError(f'File not found: {src}')

        def _generate_fx_params_h():
            target = 'SoundEnginePlugin/ProjectNameFXParams.h'
            dst = _copy_template(target)
            util.substitute_lines_in_file(self.__generate_ids(), dst, '// [ParameterID]', '// [/ParameterID]')
            util.substitute_lines_in_file(self.__generate_declarations(rtpc=True), dst, '// [RTPCDeclaration]',
                                          '// [/RTPCDeclaration]')
            util.substitute_lines_in_file(self.__generate_declarations(rtpc=False), dst, '// [NonRTPCDeclaration]',
                                          '// [/NonRTPCDeclaration]')

        def _generate_fx_params_cpp():
            target = 'SoundEnginePlugin/ProjectNameFXParams.cpp'
            dst = _copy_template(target)
            util.substitute_lines_in_file(self.__generate_init(), dst, '// [ParameterInitialization]',
                                          '// [/ParameterInitialization]')
            util.substitute_lines_in_file(self.__generate_read_bank_data(), dst, '// [ReadBankData]',
                                          '// [/ReadBankData]')
            util.substitute_lines_in_file(self.__generate_set_parameter(), dst, '// [SetParameters]',
                                          '// [/SetParameters]', withindent=False)

        def _generate_wwise_plugin_h():
            target = 'WwisePlugin/ProjectNamePlugin.h'
            dst = _copy_template(target)
            util.substitute_lines_in_file(self.__generate_property_name_declaration(), dst, '// [PropertyNames]',
                                          '// [/PropertyNames]')

        def _generate_wwise_plugin_cpp():
            target = 'WwisePlugin/ProjectNamePlugin.cpp'
            dst = _copy_template(target)
            util.substitute_lines_in_file(self.__generate_property_name_definition(), dst, '// [PropertyNames]',
                                          '// [/PropertyNames]')
            util.substitute_lines_in_file(self.__generate_write_bank_data(), dst, '// [WriteBankData]',
                                          '// [/WriteBankData]')

        def _generate_wwise_xml():
            target = 'WwisePlugin/ProjectName.xml'
            dst = _copy_template(target)
            util.substitute_lines_in_file(self.__generate_parameter_gui(), dst, '<!-- [ParameterGui] -->',
                                          '<!-- [/ParameterGui] -->')
            util.substitute_lines_in_file(self.__generate_platform_support(), dst, '<!-- [PlatformSupport] -->',
                                          '<!-- [/PlatformSupport] -->')

        def _generate_doc():
            for param in self.parameters.values():
                param.dump_parameter_doc(self.pathMan.docsDir)

        _generate_fx_params_h()
        _generate_fx_params_cpp()
        _generate_wwise_plugin_h()
        _generate_wwise_plugin_cpp()
        _generate_wwise_xml()
        _generate_doc()

    def __generate_ids(self):
        lines = []
        for i, param in enumerate(self.parameters.values()):
            lines.append(param.generate_param_id())
        lines.append(f'static constexpr AkUInt32 NUM_PARAMS = {len(self.parameters)};')
        return auto_add_line_end(lines)

    def __generate_declarations(self, rtpc=True):
        lines = []
        for param in self.parameters.values():
            if param.rtpc == rtpc:
                lines.append(param.generate_declaration())
        return auto_add_line_end(lines)

    def __generate_init(self):
        lines = []
        for param in self.parameters.values():
            lines.append(param.generate_init())
        return auto_add_line_end(lines)

    def __generate_read_bank_data(self):
        lines = []
        for param in self.parameters.values():
            lines.append(param.generate_read_bank_data())
        return auto_add_line_end(lines)

    def __generate_set_parameter(self):
        lines = []
        for param in self.parameters.values():
            lines.append(param.generate_set_parameter())
        return auto_add_line_end(lines)

    def __generate_property_name_declaration(self):
        lines = []
        for param in self.parameters.values():
            lines.append(param.generate_property_name_declaration())
        return auto_add_line_end(lines)

    def __generate_property_name_definition(self):
        lines = []
        for param in self.parameters.values():
            lines.append(param.generate_property_name_definition())
        return auto_add_line_end(lines)

    def __generate_write_bank_data(self):
        lines = []
        for param in self.parameters.values():
            lines.append(param.generate_write_bank_data())
        return auto_add_line_end(lines)

    def __generate_parameter_gui(self):
        lines = []
        for param in self.parameters.values():
            lines.extend(param.generate_parameter_gui())
        return auto_add_line_end(lines)

    def __generate_platform_support(self):
        return auto_add_line_end(self.pluginInfo.generate_platform_support())



import tomllib
import os.path as osp
from distutils.dir_util import copy_tree
from distutils.file_util import copy_file
from pathlib import Path

__all__ = [
    'overwrite_copy',
    'path_is_under',
    'load_toml',
    'replace_in_basename',
]


def overwrite_copy(src: str, dst: str):
    if osp.isfile(src):
        copy_file(src, dst)

    if osp.isdir(src):
        copy_tree(src, dst)


def path_is_under(child: str, parent: str) -> bool:
    return Path(parent) in Path(child).parents


def load_toml(path):
    with open(path, 'rb') as f:
        toml_content = tomllib.load(f)
    return toml_content


def replace_in_basename(path: str, old: str, new: str):
    return osp.join(osp.dirname(path), osp.basename(path).replace(old, new))



import logging
import os.path as osp
import platform
import subprocess
import glob

# 3rd party
import kkpyutil as util

# project
import wpe.util as wpe_util
from wpe.pathman import PathMan
from wpe.wp_wrapper import WpWrapper
from wpe.parameter import ParameterGenerator
from wpe.hook_processor import HookProcessor


class Worker:
    def __init__(self, args, path_man=None):
        self.args = args

        self.wpWrapper = WpWrapper()
        self.pathMan = path_man

        self.terminatedWwise = False

    @staticmethod
    def get_platform_worker(args):
        system = platform.system()
        if system == 'Windows':
            return WindowsWorker(args)
        raise NotImplementedError(f'Not implemented for this platform: {system}')

    def _lazy_init_pathman(self):
        self.pathMan = self.pathMan or PathMan()

    def main(self):
        self.wpWrapper.validate_env()

        if self.args.wp:
            return self.wp()

        if self.args.new:
            return self.new()

        self._lazy_init_pathman()

        if self.args.initWpe:
            return self.init_wpe()

        hook_processor = HookProcessor(self.pathMan, self.args.configuration, self.args.withHooks)
        if self.args.premake:
            hook_processor.process_pre_hook('premake')
            self.premake()
            hook_processor.process_post_hook('premake')

        if self.args.generateParameters:
            hook_processor.process_pre_hook('generate_parameters')
            self.generate_parameters()
            hook_processor.process_post_hook('generate_parameters')

        if self.args.build:
            hook_processor.process_pre_hook('build')
            self.build()
            hook_processor.process_post_hook('build')

        if self.args.pack:
            hook_processor.process_pre_hook('pack')
            self.pack()
            hook_processor.process_post_hook('pack')

    def wp(self):
        logging.info('Run wp.py')
        self.wpWrapper.wp(self.args.wp)

    def new(self):
        logging.info('Create new project')
        self.wpWrapper.new()
        self.init_wpe()
        self.premake()
        self.generate_parameters()
        logging.info('Next step: implement your plugin, build with hooks by command: wpe -b -H')

    def init_wpe(self):
        logging.info('Initialize wpe project config')
        self._lazy_init_pathman()

        wpe_util.overwrite_copy(
            osp.join(self.pathMan.templatesDir, '.wpe'),
            osp.join(self.pathMan.configDir)
        )

    def premake(self):
        logging.info('Premake project')
        self.wpWrapper.premake(self.args.platform)

    def generate_parameters(self):
        parameter_manager = ParameterGenerator(self.pathMan, is_forced=self.args.force)
        parameter_manager.main()
        self.wpWrapper.build('Documentation')

    def build(self):
        self._terminate_wwise()
        self._build()
        self._reopen_wwise()

    def pack(self):
        def _collect_packages(_output_dir):
            util.remove_tree(_output_dir)
            for pkg in glob.iglob(osp.join(self.pathMan.root, f'{self.pathMan.pluginName}*.tar.xz')):
                util.move_file(pkg, _output_dir, isdstdir=True)
            util.move_file(osp.join(self.pathMan.root, 'bundle.json'), _output_dir, isdstdir=True)

        def _zip_bundle(_output_dir):
            util.zip_dir(_output_dir)

        logging.info('Package plugin and generate bundle')
        version_code, build_number = self.wpWrapper.wwiseVersion.rsplit('.', 1)
        if osp.isfile(self.pathMan.buildNumberFile):
            build_number = util.load_json(self.pathMan.buildNumberFile) + 1
        else:
            build_number = 1
        plugin_version = f'{version_code}.{build_number}'
        output_dir = osp.join(self.pathMan.distDir, f'{self.pathMan.pluginName}_v{version_code}_Build{build_number}')
        self.wpWrapper.package('Common', '-v', plugin_version)
        self.wpWrapper.package('Documentation', '-v', plugin_version)
        self.wpWrapper.package('Windows_vc160', '-v', plugin_version)
        self.wpWrapper.package('Authoring', '-v', plugin_version)
        self.wpWrapper.generate_bundle('-v', plugin_version)
        _collect_packages(output_dir)
        _zip_bundle(output_dir)
        util.save_json(self.pathMan.buildNumberFile, build_number)
        logging.info(f'Saved to {output_dir}')

    def _build(self):
        logging.info('Build authoring plugin')
        self.wpWrapper.build('Authoring', '-c', self.args.configuration, '-x', 'x64', '-t', 'vc160')

    def _terminate_wwise(self):
        raise NotImplementedError('subclass it')

    def _reopen_wwise(self):
        raise NotImplementedError('subclass it')


class WindowsWorker(Worker):
    def __init__(self, args):
        super().__init__(args)

    def _build(self):
        super()._build()
        logging.info('Build shared plugin')
        self.wpWrapper.build('Windows_vc160', '-c', self.args.configuration, '-x', 'x64', '-t', 'vc160')

    def _terminate_wwise(self):
        if self.args.force:
            cmd = ['taskkill', '/IM', 'wwise.exe', '/F']
            try:
                util.run_cmd(cmd)
                self.terminatedWwise = True
            except subprocess.CalledProcessError:
                pass

    def _reopen_wwise(self):
        if self.args.force and self.terminatedWwise:
            wwise_exe = osp.join(self.wpWrapper.wwiseRoot, 'Authoring/x64/Release/bin/Wwise.exe')
            util.run_daemon([wwise_exe])



import argparse

from wpe.core import Worker


def main():
    parser = argparse.ArgumentParser(
        prog='Plugin dev ci build tool',
        add_help=True,
        epilog='Wrapper of `wp.py`. Easy to premake, build, deploy and distribute wwise plugins.',
        formatter_class=argparse.RawTextHelpFormatter
    )

    command_group = parser.add_mutually_exclusive_group()
    command_group.add_argument(
        '--wp',
        action='store',
        dest='wp',
        nargs='*',
        default=[],
        required=False,
        help='Pass arguments to wp.py. Example: --wp new'
    )

    command_group.add_argument(
        '-n',
        '--new',
        action='store_true',
        dest='new',
        default=False,
        required=False,
        help='Create a new plugin project. Will auto init wpe config.'
    )

    command_group.add_argument(
        '-i',
        '--init-wpe',
        action='store_true',
        dest='initWpe',
        default=False,
        required=False,
        help='Initialize wpe project config for existing plugin project.'
    )

    command_group.add_argument(
        '-p',
        '--premake',
        action='store_true',
        dest='premake',
        default=False,
        required=False,
        help='Premake authoring project.'
    )

    command_group.add_argument(
        '-gp',
        '--generate-parameters',
        action='store_true',
        dest='generateParameters',
        default=False,
        required=False,
        help='Generate source code with parameters. Define parameters in `$PROJECT_ROOT/.wpe/wpe_parameters.toml`.'
    )

    command_group.add_argument(
        '-b',
        '--build',
        action='store_true',
        dest='build',
        default=False,
        required=False,
        help='Build project.'
    )

    command_group.add_argument(
        '-P',
        '--pack',
        action='store_true',
        dest='pack',
        default=False,
        required=False,
        help='Package plugin.'
    )

    parser.add_argument(
        '-H',
        '--with-hooks',
        action='store',
        dest='withHooks',
        nargs='*',
        default=[],
        help='''Execute command with hooks. You can specify hooks to execute. e.g. -H pre_build post_build. leave empty to execute all hooks.
Hooks should:
- be placed in `$PROJECT_ROOT/.wpe/hooks` with name `pre_<command>.py` or `post_<command>.py`.
  - supported commands: `premake`, `generate_parameters`, `build`, `pack`
- define a function with name `main`, accept followed arguments (or use `**kwargs`):
  - proj_root: project root path
  - build_config: build configuration (Debug, Profile, Release)
  - plugin_name: plugin name, which is used to glob plugin files'''
    )

    parser.add_argument(
        '-plt',
        '--platform',
        action='store',
        choices=('Android', 'Authoring', 'Authoring_Windows', 'Authoring_Linux', 'Authoring_Mac', 'iOS', 'Linux', 'LinuxAuto', 'Mac', 'NX', 'PS4', 'PS5', 'QNX', 'tvOS', 'Windows_vc160', 'Windows_vc170', 'WinGC', 'XboxOneGC', 'XboxSeriesX'),
        dest='platform',
        default='Authoring',
        required=False,
        help='Platform to premake. Default value is Authoring.'
    )
    parser.add_argument(
        '-c',
        '--configuration',
        action='store',
        choices=('Debug', 'Profile', 'Release'),
        dest='configuration',
        default='Debug',
        required=False,
        help='Configuration to build (Debug, Release, Profile). Default value is Debug.'
    )
    parser.add_argument(
        '-f',
        '--force',
        action='store_true',
        dest='force',
        required=False,
        default=False,
        help='''Force operation. This argument is compatible with -gp or -b.
    For -gp, it will overwrite existing source files.
    For -b, it will terminate Wwise process and copy plugin files, then reopen Wwise.'''
    )

    parsed_args = parser.parse_args()
    worker = Worker.get_platform_worker(parsed_args)
    worker.main()


if __name__ == '__main__':
    main()



import os
import os.path as osp

import kkpyutil as util


class WpWrapper:
    def __init__(self):
        self.wwiseRoot: str = os.getenv('WWISEROOT')
        self.wwiseSDKRoot: str = os.getenv('WWISESDK')
        self.wwiseVersion: str = self._load_wwise_version()
        self.wpScriptDir = osp.join(self.wwiseRoot, 'Scripts/Build/Plugins')

        self.subcommands = (
            'build',
            'generate_bundle',
            'new',
            'package',
            'premake',
        )

    def _load_wwise_version(self) -> str:
        install_entry = util.load_json(osp.join(self.wwiseRoot, 'install-entry.json'))
        version = install_entry['bundle']['version']
        return f'{version["year"]}.{version["major"]}.{version["minor"]}.{version["build"]}'

    def validate_env(self):
        if self.wwiseRoot is None:
            raise EnvironmentError(f'Unknown env variable: WWISEROOT\n  - Try setting environment variables in Wwise '
                                   f'Launcher')

        if self.wwiseSDKRoot is None:
            raise EnvironmentError(f'Unknown env variable: WWISESDK\n  - Try setting environment variables in Wwise '
                                   f'Launcher')

        if not osp.isfile((wp := osp.join(self.wpScriptDir, 'wp.py'))):
            raise FileNotFoundError(f'"{wp}" not found.')

        for subcommand in self.subcommands:
            if not osp.isfile((subcommand_file := osp.join(self.wpScriptDir, f'{subcommand}.py'))):
                raise FileNotFoundError(f'Subcommand "{subcommand_file}" not found.')

    def wp(self, args):
        subcommand = args[0]
        if subcommand not in self.subcommands:
            raise ValueError(f'Unknown subcommand: {subcommand}')
        return self.run(subcommand, *args[1:])

    def build(self, *args):
        res = self.run('build', *args)
        if res != 0:
            raise RuntimeError(f'Build failed. Exit code: {res}')
        return res

    def generate_bundle(self, *args):
        return self.run('generate_bundle', *args)

    def new(self, *args):
        return self.run('new', *args)

    def package(self, *args):
        util.lazy_prepend_sys_path([self.wpScriptDir])
        import wpe.wp_subcommand_patched.package as wsp_package
        res = wsp_package.run(args)
        util.lazy_remove_from_sys_path([self.wpScriptDir])
        return res

    def premake(self, *args):
        return self.run('premake', *args)

    def run(self, subcommand, *args):
        subcommand_module = util.safe_import_module(osp.basename(subcommand), self.wpScriptDir)
        return subcommand_module.run(args)



#!/usr/bin/env python3

"""
The content of this file includes portions of the AUDIOKINETIC Wwise Technology
released in source code form as part of the SDK installer package.

Commercial License Usage

Licensees holding valid commercial licenses to the AUDIOKINETIC Wwise Technology
may use this file in accordance with the end user license agreement provided
with the software or, alternatively, in accordance with the terms contained in a
written agreement between you and Audiokinetic Inc.

  Copyright (c) 2021 Audiokinetic Inc.
"""

import argparse
import errno
import json
from glob import glob
import os
import shutil
import subprocess
import sys
import tarfile
import re
import codecs
from common.constant import PLUGIN_NAME, PROJECT_ROOT, WWISE_ROOT, XZ_UTILS
from common.platform import *
from common.registry import platform_registry, get_supported_platforms, is_documentation, is_authoring_target
from common.util import exit_with_error, strip_comments
from common.version import VersionArgParser

SUPPORTED_PLATFORMS = get_supported_platforms("package")
DEFAULT_ADDITIONAL_ARTIFACT_FILE = "additional_artifacts.json"

def find_artifacts(args):
    # search for artifacts using the glob patterns
    artifacts_found = []
    artifacts_found_in_project = []

    platform_info = platform_registry.get(args.platform)
    # [wp-enhanced] exclude pdb and debug artifacts
    debug_artifacts_folders = {'Debug', 'Debug(StaticCRT)', 'Debug-iphoneos', 'Debug-iphonesimulator'}
    for artifact_glob in (pattern for pattern in platform_info.package.artifacts if not pattern.endswith(".pdb")):
        for artifact in glob(os.path.join(WWISE_ROOT, artifact_glob.format(plugin_name=PLUGIN_NAME))):
            sections = set(artifact.split(os.path.sep))
            if sections & debug_artifacts_folders:
                continue
            artifacts_found.append(artifact)
            print("Found {}".format(artifact))
    # [/wp-enhanced]

    # validate additional artifacts
    for artifact_glob in args.additional_artifacts:
        artifacts = glob(os.path.join(WWISE_ROOT, os.path.normpath(artifact_glob)))
        if not artifacts:
            exit_with_error("Could not find additional artifact at {}".format(artifact_glob))

        for artifact in artifacts:
            artifacts_found.append(artifact)
            print("Added {}".format(artifact))

    # do the same with additional artifacts coming from a json file
    if args.additional_artifacts_file:
        try:
            with codecs.open(os.path.join(PROJECT_ROOT, args.additional_artifacts_file), "r", "utf-8") as f:
                additional_artifacts = json.loads(strip_comments(f.read()))
                platform_name = "Authoring" if "Authoring" in platform_info.name else platform_info.name
                if platform_name in additional_artifacts:
                    platform_additional_artifacts = additional_artifacts.get(platform_name)
                    for entry in platform_additional_artifacts:
                        if isinstance(entry, dict):
                            # we have a dictionary of destination -> source globs that are relative to PROJECT_ROOT
                            for dest, sources in entry.items():
                                for artifact_glob in sources:
                                    for artifact in glob(os.path.join(PROJECT_ROOT, os.path.normpath(artifact_glob))):
                                        artifacts_found_in_project.append((artifact, dest))
                                        print("Added {}".format(artifact))
                        else:
                            # we have a source glob that is relative to WWISE_ROOT
                            for artifact in glob(os.path.join(WWISE_ROOT, os.path.normpath(entry))):
                                artifacts_found.append(artifact)
                                print("Added {}".format(artifact))
        except IOError:
            if args.additional_artifacts_file != DEFAULT_ADDITIONAL_ARTIFACT_FILE:
                # only report error if user specified a non-default value
                exit_with_error("Missing additional artifacts file {} at {}".format(args.additional_artifacts_file, PROJECT_ROOT))
        except ValueError as e:
            exit_with_error("Invalid additional artifacts file {} at {}\n{}".format(args.additional_artifacts_file, PROJECT_ROOT, str(e)))

    return (artifacts_found, artifacts_found_in_project)

def run(argv):
    # parse the command line
    parser = argparse.ArgumentParser(description="Audiokinetic Inc. packaging tool for plugins")
    parser.add_argument("-v", "--version", action=VersionArgParser, help="version of the package (formatted as 'year.major.minor.build')")
    parser.add_argument("platform", metavar="platform", choices=SUPPORTED_PLATFORMS, help="platform to package ({})".format(", ".join(SUPPORTED_PLATFORMS)))
    parser.add_argument("-a", "--additional-artifacts", nargs="*", default=[], help="path to additional artifacts to package (must be relative to the root of your Wwise installation, supports glob patterns)")
    parser.add_argument("-f", "--additional-artifacts-file", default=DEFAULT_ADDITIONAL_ARTIFACT_FILE, help="path to a JSON file listing the paths to additional artifacts to package for each platform, defaults to {}".format(DEFAULT_ADDITIONAL_ARTIFACT_FILE))
    parser.add_argument("-c", "--copy-artifacts", action="store_true", help="copy artifacts instead of packaging them, this option only applies to additional artifact files with destination -> sources entries")
    args = parser.parse_args(argv)

    platform_info = platform_registry.get(args.platform)

    if args.copy_artifacts:
        # run in copy mode, copy artifacts from the PROJECT_ROOT to the WWISE_ROOT
        print("Copying artifacts for {}...".format(platform_info.name))

        artifacts_found, artifacts_found_in_project = find_artifacts(args)
        if not artifacts_found and not artifacts_found_in_project:
            exit_with_error("Nothing to copy!")

        for (artifact, dest) in artifacts_found_in_project:
            dest = os.path.join(WWISE_ROOT, os.path.normpath(dest))
            try:
                os.makedirs(dest)
            except OSError as e:
                if e.errno != errno.EEXIST: # for python2 compatibility, ideally we would use the exist_ok param in makedirs
                    raise

            print("Copied {} to {}".format(artifact, dest))
            shutil.copy(artifact, dest)
    else:
        # run in packaging mode, compress everything into an archive
        if not args.version:
            exit_with_error("'Version' argument needs to be specified using -v or --version, -h for more details.")

        print("Packaging {} for {}...".format(PLUGIN_NAME, platform_info.name))

        artifacts_found, artifacts_found_in_project = find_artifacts(args)
        if not artifacts_found and not artifacts_found_in_project:
            exit_with_error("Nothing to package!")

        formatted_plugin_version = "v{}.{}.{}_Build{}".format(
            args.version.year, args.version.major, args.version.minor, args.version.build)

        if is_documentation(platform_info.name):
            formatted_platforms = ["Authoring.{}".format(platform_info.name)]
        elif is_authoring_target(platform_info.name):
            formatted_platforms = [{
                                       "Authoring_Windows": "{}_{}.x64",
                                       "Authoring_Mac": "{}_{}.macosx_gmake",
                                       "Authoring_Linux": "{}_{}.linux_gmake"
                                   }.get(platform_info.name, "{}").format(platform_info.name, config) for config in ["Debug", "Release"]
                                   ]
        else:
            formatted_platforms = ["SDK.{}".format(platform_info.name)]

        compressed_archive_names = [
            "{}_{}_{}.tar.xz".format(
                PLUGIN_NAME,
                formatted_plugin_version,
                formatted_platform
            ) for formatted_platform in formatted_platforms
        ]

        output_name_re = re.compile(r"^" + PLUGIN_NAME + r"_v\d+\.\d+\.\d+_Build\d+_(Authoring|Authoring_Windows|Authoring_Linux|Authoring_Mac)_(?P<config>Debug|Release)(?:\.(.*))?\.tar\.xz$")

        def is_authoring_debug_package(output_name):
            match = re.match(output_name_re, output_name)
            return False if match is None else match.group("config") == "Debug"

        def is_authoring_release_package(output_name):
            match = re.match(output_name_re, output_name)
            return False if match is None else match.group("config") == "Release"

        def is_data_artifact(artifact):
            return os.path.join("Authoring", "Data") in artifact or os.path.join("Authoring", "Help") in artifact

        def is_debug_artifact(artifact):
            artifact_re = re.compile(r"([/\\])Debug\1")
            return bool(artifact_re.search(artifact))

        # [wp-enhanced] add artifacts recursively, avoid adding empty folders
        def archive_artifacts(output_name, write_mode):
            def need_add(_artifact):
                if is_authoring_debug_package(output_name):
                    if not is_debug_artifact(_artifact) or is_data_artifact(_artifact):
                        return False
                elif is_authoring_release_package(output_name) and is_debug_artifact(_artifact):
                    return False
                return True

            artifacts_dst_pair: list[tuple[str, str]] = []
            for artifact in artifacts_found:
                if need_add(artifact):
                    print("Compressing {}...".format(artifact))
                    artifacts_dst_pair.append((artifact, os.path.relpath(artifact, WWISE_ROOT)))
            for (artifact, dest) in artifacts_found_in_project:
                if need_add(artifact):
                    print("Compressing {}...".format(artifact))
                    artifacts_dst_pair.append((artifact, os.path.join(dest, os.path.basename(artifact))))

            if not artifacts_dst_pair:
                return
            with tarfile.open(output_name, write_mode, format=tarfile.GNU_FORMAT) as tar:
                for artifact, dest in artifacts_dst_pair:
                    tar.add(artifact, dest, recursive=True)
        # [/wp-enhanced]

        for compressed_archive_name in compressed_archive_names:
            try:
                archive_artifacts(compressed_archive_name, "w:xz")
            except tarfile.CompressionError:
                # xz compression isn't supported on older versions of tarfile
                archive_name = compressed_archive_name[:-3]
                archive_artifacts(archive_name, "w")

                res = subprocess.Popen([XZ_UTILS, "-zf", archive_name]).wait()
                if res != 0:
                    return res

            print("Wrote {}".format(compressed_archive_name))

    return 0

if __name__ == "__main__":
    sys.exit(run(sys.argv[1:]))
